/*jslint node:true*/
module.exports = function (grunt) {
    "use strict";

    // Load all grunt tasks.
    require('load-grunt-tasks')(grunt);

    grunt.initConfig({

        jslint: {
            bin: {
                src: [
                    "bin/**.js"
                ],

                options: {
                    shebang: true
                }
            },
            src: {
                src: [
                    "*{.js,.json}"
                ]
            },
            coverage: {
                src: [
                    "coverage/*.js"
                ]
            },
            test: {
                src: [
                    "test/**{.js,.json}"
                ],

                //
                // mocha specific jslint directions could be added here.
                //

                exclude: [
                    "test/fs/**"
                ]
            }
        },

        mochaTest: {
            test: {
                options: {
                    require: "coverage/blanket",
                    reporter: "spec"
                },

                src: ["test/*.js"]
            },

            "html-coverage": {
                options: {
                    reporter: "html-cov",
                    quiet: true,
                    captureFile: "coverage/coverage.html"
                },
                src: ["test/*.js"]
            },

            // The travis-cov reporter will fail the tests if the
            // coverage falls below the threshold configured in package.json
            "travis-coverage": {
                options: {
                    reporter: "travis-cov"
                },
                src: ["test/*.js"]
            }
        },

        watch: {
            test: {
                files: [
                    "*.js",
                    "bin/*.js",
                    "coverage/*.js",
                    "test/*.js"
                ],
                tasks: [
                    "test"
                ]
            }
        },

        clean: {
            "coverage": "coverage/coverage.html"
        }

    });

    grunt.registerTask("test", [
        "jslint",
        "mochaTest"
    ]);

    grunt.registerTask("default", [
        "clean",
        "test"
    ]);

};
