# node-epages6

This is a helper for ePages 6 written in JavaScript for node.js. It provides the functionality to
pack the JavaScript, Less and CSS files of a store as well as creating a minified version of the entire
*~/lib* directory.
It also searches for missing test cases and reports the current health of our test environment.


## Installation

You can install this node module via `npm`. Because this module is not available in public
repositories, you will need a tarball containing the module. Locate this tarball, change into the
containing directory and issue the following command:

    npm install -g node-epages6.tar.gz

After installation make sure to run `npm install` from within the node-epages6 directory.

## Synposis & Usage

    usage: epnode6.js [-h] [-v] [-l] {minify,pack,builddoc,testcases} ...

    ePages 6 node.js helper

    Optional arguments:
      -h, --help            Show this help message and exit.
      -v, --version         Show program's version number and exit.
      -l, --extended-log    Activate extended logging and stack tracing

    subcommands:
      {minify,pack,builddoc,testcases}

###Subcommands
epnode6 is divided into four subcommands which can be accessed separately.
####Minify
    usage: epnode6.js minify [-h] -v STORE_VERSION -t STORE_TYPE [-w WEBROOT]
                         [-f FILTER]


    Minify all JavaScript and CSS files within the lib folder.

    Optional arguments:
      -h, --help            Show this help message and exit.
      -v STORE_VERSION, --store-version STORE_VERSION
                            Store version e.g. 6.17.0
      -t STORE_TYPE, --store-type STORE_TYPE
                            Store type e.g. Store
      -w WEBROOT, --webroot WEBROOT
                            Epages webroot defaults to $EPAGES_WEBROOT
      -f FILTER, --filter FILTER
                            Filter files to be minified e.g. /**/*.js

    Example: epnode minify -t Store -v 6.17.0 -f /**/*.ca.json

####Pack
    usage: epnode6.js pack [-h] [-j] [-c] [-l] -p {bo,sf,mobilesf,editor,pageeditor,testsuite,sf_appointment}
                           -t STORE_TYPE -v STORE_VERSION [-w WEBROOT]


    Create Packages for JavaScript and CSS files from the specified profile.

    Optional arguments:
      -h, --help            Show this help message and exit.
      -j, --js              Create JavaScript package from profile ['bo', 'sf',
                            'mobilesf', 'editor', 'testsuite', 'sf_appointment']
      -c, --css             Create CSS package from profile ['sf', 'mobilesf',
                            'editor', 'testsuite']
      -l, --less            Create LESS package from profile ['bo', 'pageeditor']
      -p {bo,sf,mobilesf,editor,pageeditor,testsuite,sf_appointment}, --profile {bo,sf,mobilesf,editor,pageeditor,testsuite,sf_appointment}
                            Profile used for packaging
      -t STORE_TYPE, --store-type STORE_TYPE
                            Store type e.g. Store
      -v STORE_VERSION, --store-version STORE_VERSION
                            Store version e.g. 6.17.0
      -w WEBROOT, --webroot WEBROOT
                            Epages webroot defaults to $EPAGES_WEBROOT

    Example: epnode pack --js --css -p sf -p mobilesf -p editor -p testsuite -t Store -v 6.17.0

####Test cases
    usage: epnode6.js testcases [-h] [-c CARTRIDGE]

    Check for the presence of testcases

    Optional arguments:
      -h, --help            Show this help message and exit.
      -c CARTRIDGE, --cartridge CARTRIDGE
                            Check if the cartridge has testcases [multi]

    Example: epnode testcases -c jquery

####Build documentation
    usage: epnode6.js builddoc [-h] -i INCLUDE [-o OUTPUT] [-c CARTRIDGE_FOLDER]

    Create the jsduck documentation.

    Optional arguments:
      -h, --help            Show this help message and exit.
      -i INCLUDE, --include INCLUDE
                            A JSON file which enumerates the files to include in
                            the documentation.
      -o OUTPUT, --output OUTPUT
                            The output folder for the documentation
      -c CARTRIDGE_FOLDER, --cartridge-folder CARTRIDGE_FOLDER
                            The folder containing the cartridges, typically
                            /srv/epages/eproot/Cartridges

    Example: epnode builddoc -i sources.json -o ./

## Examples

Minify all files under */srv/epages/eproot/Shared/WebRoot/StoreTypes/6.16.1/Store/lib* and
store them under */srv/epages/eproot/Shared/WebRoot/StoreTypes/6.16.1/Store/lib-min*. CVS files,
documentation files and test files are excluded.

    epnode6.js minify -t Store -v 6.17.1

Create a package for the LESS and JavaScript files for the `bo` profile.

    epnode6 pack -p bo -t Store -v 6.16.1 --js --less

Create a package for the CSS and JavaScript files for the `sf`, `mobilesf`, `editor` and `testsuite` profile.

    epnode6 pack -p sf -p mobilesf -p editor -p testsuite -t Store -v 6.16.1 --js --css