/*jslint indent:4,node:true,nomen:true,stupid:true,regexp:true*/
(function () {

    'use strict';

    var _, fs, os, grunt, path, async, cp, chownr, isFile, ignoreFile, filterFile, commons;

    _ = require('underscore');
    fs = require('fs.extra');
    os = require('os');
    grunt = require('grunt');
    path = require('path');
    async = require('async');
    cp = require('child_process');
    chownr = require('chownr');
    commons = require('./commons');

    isFile = function (filePath) {
        return fs
            .lstatSync(filePath)
            .isFile();
    };

    ignoreFile = function (filePath, options) {
        // Options is a received argument or the contextual object
        var ignore = options.ignore,
            result = true;

        // Check is a file
        if (!isFile(filePath)) {
            result = false;
        } else if (ignore instanceof RegExp) {
            // Check does not match ignore regualar expression
            result = !ignore.test(filePath);
        } else if (typeof ignore === 'string' || _.isArray(ignore)) {
            // Check does not match ignore minimatch expression
            result = !grunt.file.isMatch(ignore, filePath);
        }

        return result;
    };

    // Filter handle
    filterFile = function (filePath, options) {
        // Options is a received argument or the contextual object
        var filter = options.filter,
            result = true;

        // Check is a file
        if (!isFile(filePath)) {
            result = false;
        } else if (filter instanceof RegExp) {
            // Check match filter regualar expression
            result = filter.test(filePath);
        } else if (typeof filter === 'string' || _.isArray(filter)) {
            // Check match filter minimatch
            result = grunt.file.isMatch(filter, filePath);
        }

        // Check ignore
        result = result ? ignoreFile(filePath, options) : result;

        return result;
    };

    // Exports minify method
    module.exports = function (source, target, options, callback) {
        var workers, q, i, files;

        workers = [];
        for (i = 0; i < os.cpus().length; i += 1) {
            workers.push({ worker: cp.fork(__dirname + '/minify-fork'), free: true });
        }

        q = async.queue(function (task, callback) {
            async.filter(workers, function (w, callback) {
                callback(w.free);
            }, function (free) {
                var w = free[0];

                w.free = false;
                w.worker.send(task);

                w.worker.once('message', function (e) {
                    if (e) {
                        callback(e);
                    } else {
                        w.free = true;
                        callback();
                    }
                });
            });
        }, workers.length);

        options = _.extend({
            ignore: /(\.tmpl\.html|\.tmpl\.js|-test|-docu|CVS)/,
            filter: /.*/
        }, options);

        // List of files to minify
        files = grunt.file.expand(source + '/**/*')
            .filter(function (filePath) {
                return filterFile(filePath, options);
            });

        async.eachLimit(files, workers.length, function (filePath, callback) {
            var relativePath = path.relative(source, filePath),
                targetFilePath = path.join(target, relativePath);

            async.series([
                function (callback) {
                    fs.mkdirp(path.dirname(targetFilePath), callback);
                },

                function (callback) {
                    q.push({
                        base: source,
                        source: filePath,
                        target: targetFilePath
                    }, callback);
                }
            ], callback);
        }, function (e) {
            if (_.isNumber(options.uid) && _.isNumber(options.gid)) {
                chownr.sync(target, options.uid, options.gid);
            }

            // cleaning up the mess
            commons.cleanUp(source);

            callback(e);
        });
    };
}());